apply plugin: 'java'

import groovy.util.slurpersupport.GPathResult;

import org.gradle.api.artifacts.*


repositories {
	mavenCentral()
}

dependencies {
//		compile (group: 'commons-logging', name: 'commons-logging', version: '1.1.1')
	compile (group: 'org.springframework', name: 'spring-context', version: '3.1.2.RELEASE')
}

task retrieveAndParsePom << {
	//get configuration
	def configuration = project.configurations.getByName('compile')

	def artifacts = new HashSet()
	
	def i = 0; // loop-counter
	
	while (true) {
		println "loop " + (i++)
		def artifactsCounter = artifacts.size
		artifacts.clear()

		def oldDependencySet = configuration.dependencies
		def newConfig = resolveConfiguration(configuration, artifacts )
		
		logger.debug("Remove already resolved dependencies ...")
		oldDependencySet.each {
			newConfig.dependencies.remove(it)
		}

		configuration  = newConfig

		if (artifactsCounter == artifacts.size) break
	}

}

def Configuration resolveConfiguration(Configuration configuration, def artifacts ) {
	def sampleDependency = configuration.dependencies.iterator().next();

	// resolve configuration to get any transitive dependency
	def resolvedConf = configuration.resolvedConfiguration
	def lenientConf = resolvedConf.lenientConfiguration
	def firstLevelDep = lenientConf.firstLevelModuleDependencies
	firstLevelDep.each() { resolveDependencies(artifacts, it)  }


	println "#artifacts: " + artifacts.size()

	def newConfig = configuration.copyRecursive()

	project.configurations.add(newConfig)

	artifacts.each {
		def depCopy = createNewDependency(sampleDependency, it.group, it.name, it.version)
		newConfig.dependencies.add(depCopy)

		if (it.extension.equals("pom")) {
			def addtionalDependecies = parsePom(sampleDependency, it);
			newConfig.dependencies.addAll(addtionalDependecies)
		}

	}

	//	artifacts.clear()

	return newConfig
}

def List<DependencyArtifact> createNewDependencyArtifactList(String name) {
	def list = new ArrayList<DependencyArtifact>()
	list.add(new DependencyArtifact() {
				String getClassifier() { return null}
				String getName() {return name}
				String getType() {return 'pom'}
				String getExtension() {return 'pom'}
				String getUrl() {return null}
			}
			)

	return list;
}


def Dependency createNewDependency (Dependency sample, String group, String name, String version) {
	def copy = sample.copy()
	copy.group = group;
	copy.name = name;
	copy.version = version;
	copy.artifacts.clear();
	copy.artifacts.addAll(createNewDependencyArtifactList(name))
	return copy;
}


def resolveDependencies(Set artifacts, ResolvedDependency d) {

	d.moduleArtifacts.each { a ->
		logger.debug( "try to resolve artifact " + a + " ... ")
		try {
			artifacts.add(['group': d.moduleGroup, 'name': d.moduleName, 'version': d.moduleVersion, 'file': a.file, 'classifier': a.classifier, 'extension': a.extension])
			logger.debug( "... resolved")
		} catch (org.gradle.api.internal.artifacts.ivyservice.ivyresolve.ArtifactNotFoundException ex) {
			logger.info "Artifact not found: " + ex.message
		}
	}
	d.children.each { c ->
		resolveDependencies(artifacts, c)
	}

}

def Set<Dependency> parsePom(Dependency sample, pomFile) {
	logger.info ("parse Pom " + pomFile.file.name)
	def newDependencies = new HashSet()

	def nodeResult = new XmlSlurper(false, true).parse(pomFile.file)
	def parentPom = nodeResult.parent

	if (parentPom.artifactId.text()) {

		def newDep = createNewDependency(sample, parentPom.groupId.text(), parentPom.artifactId.text(), parentPom.version.text())
		newDependencies.add(newDep)
	}

	def dependencyNodes = nodeResult.'**'.grep { it.name().equals('dependency')}
	dependencyNodes.each {depNode ->
		if (depNode.version.text() != '') {
			def groupId = (depNode.groupId.text().contains('$') ? replacePlaceHolder(nodeResult, depNode.groupId.text()) : depNode.groupId.text())
			def artifactId = (depNode.artifactId.text().contains('$') ? replacePlaceHolder(nodeResult, depNode.artifactId.text()) : depNode.artifactId.text())
			def version = (depNode.version.text().contains('$') ? replacePlaceHolder(nodeResult, depNode.version.text()) : depNode.version.text())
			def newDep = createNewDependency(sample, groupId, artifactId, version)
			newDependencies.add(newDep)
		} else {
			logger.debug( "Version number of dependency " + depNode + " is not specified. The dependency is declared within the section 'dependencyManagement' of the same pom or a parent-POM.")
		}

	}

	return newDependencies
}

def String replacePlaceHolder(GPathResult rootNode, String template) {
	def propMap = new Map() {
				def out = null
				Object get(Object key) {
					if (key.equals('out')) {
						return out
					}
					if ('[groupId, version]'.contains(key)) {
						// It is possible, that a dependency igroupId and version
						if (rootNode.'*'.grep{it.name().equals(key)}) {
							// read the value of node
							def list = rootNode.'*'.grep{it.name().equals(key)}
							assert list.size == 1
							return list.get(0).text()
						}
						if (rootNode.parent.'*'.grep{it.name().equals(key)}) {
							def list = rootNode.parent.'*'.grep{it.name().equals(key)}
							assert list.size == 1
							return list.get(0).text()
						}
					}
					if (rootNode.properties.'*'.grep{it.name().equals(key)}) {
						// key is a value of a property that is defined within the POM
						def list = rootNode.properties.'*'.grep{it.name().equals(key)}
						assert list.size == 1
						return list.get(0).text()
					}
					return rootNode
				}
				boolean containsValue(Object obj) {throw new UnsupportedOperationException()}
				Object put(java.lang.Object obj1, java.lang.Object obj2){
					if (obj1.equals('out')) {
						out = obj2
					}
				}
				boolean isEmpty(){return false}
				Object remove(java.lang.Object obj){throw new UnsupportedOperationException()}
				int size(){throw new UnsupportedOperationException()}
				Set keySet(){throw new UnsupportedOperationException()}
				Set entrySet(){throw new UnsupportedOperationException()}
				boolean containsKey(java.lang.Object obj){throw new UnsupportedOperationException()}
				Collection values(){throw new UnsupportedOperationException()}
				void clear(){throw new UnsupportedOperationException()}
				void putAll(java.util.Map map){throw new UnsupportedOperationException()}


			}
	def engine = new groovy.text.SimpleTemplateEngine()
	def result = engine.createTemplate(template).make(propMap).toString()
	if (result.startsWith('4.0.0org')) {
		throw new IllegalStateException(template)
	}
	return result
}

